from enum import Enum
from typing import Optional
from uuid import UUID

from serenity_types.utils.serialization import CamelModel


class AssetType(Enum):
    """
    Simple classification of assets.
    """

    CURRENCY = 'CURRENCY'
    """
    Fiat currency, e.g. EUR.
    """

    TOKEN = 'TOKEN'
    """
    Generic tokenized assets.
    """

    PEGGED_TOKEN = 'PEGGED_TOKEN'
    """
    A tokenized asset whose price is linked to an exposure.
    """

    WRAPPED_TOKEN = 'WRAPPED_TOKEN'
    """
    A tokenized asset which represents a claim on a token, typically on another Network.
    """

    FUTURE = 'FUTURE'
    """
    An exchange-listed futures contract.
    """

    LISTED_OPTION = 'LISTED_OPTION'
    """
    An exchange-listed option.
    """

    OTC_OPTION = 'OTC_OPTION'
    """
    An OTC option contract.
    """

    INDEX = 'INDEX'
    """
    A basket of other assets.
    """

    STRATEGY = 'STRATEGY'
    """
    A multi-leg asset composed of positions in other assets.
    """


class Asset(CamelModel):
    """
    Base class for all financial assets tracked in Serenity.
    """

    asset_id: UUID
    """
    Unique, immutable ID for this asset. Symbols can change over time,
    but asset ID's are stable.
    """

    asset_type: AssetType
    """
    Basic classification of this asset. Based on the type, sub-classes of
    the Asset may carry additional details.
    """

    symbol: str
    """
    Serenity's unique symbol for this asset, e.g. tok.usdc.ethereum.
    """

    native_symbol: Optional[str]
    """
    Whatever is the issuer's symbol for this asset. For tokens this is typically the token smart contract symbol
    or native blockchain token symbol, e.g. ETH or DAI.
    """

    display_name: str
    """
    Human-readable name for this asset.
    """



from datetime import date, time
from enum import Enum
from typing import Optional
from uuid import UUID

from serenity_types.refdata.asset import Asset
from serenity_types.utils.serialization import CamelModel


class SettlementType(Enum):
    CASH = 'CASH'
    """
    Derivative contract settled in fiat or stablecoins.
    """

    PHYSICAL = 'PHYSICAL'
    """
    Derivative contract settled in an asset other than fiat or stablecoins.
    """


class PayoffType(Enum):
    LINEAR = 'LINEAR'
    """
    Linear payoff that follows the price movement of the underlier.
    """

    INVERSE = 'INVERSE'
    """
    Non-linear payoff that moves opposite the underlying price movements.
    """


class DerivativeAsset(Asset):
    """
    A listed or OTC derivative contract.
    """

    underlier_asset_id: UUID
    """
    The underlying asset for this derivative contract, e.g. BTC (tok.btc.bitcoin).
    Note we will be switching to the Exposure UUID instead in a future release (e.g. tok.btc),
    once the reference data is available.
    """

    reference_index_id: Optional[UUID]
    """
    The specific index, e.g. Deribit BTC Index, used to get a fair price for the underlying at settlement time.
    """

    contract_size: float
    """
    Size of the contract in qty of underlying.
    """

    settlement_asset_id: UUID
    """
    The asset that this derivatives settles in, e.g. on Deribit, CASH settled, it might be USD.
    Note we will be switching to the Exposure UUID instead in a future release.
    """

    settlement_type: SettlementType
    """
    Whether this contract settles in cash or in the underlying itself.
    """


class ListedDerivative(DerivativeAsset):
    """
    An exchange-listed derivative contract.
    """

    exchange_id: UUID
    """
    The exchange on which this contract is listed.
    """


class Expiry(CamelModel):
    expiration_date: date
    """
    The local expiration date in the contract specification; exact datetime requires a timezone.
    """

    expiration_time: time
    """
    The local expiration time in the contract specification; exact datetime requires a timezone.
    """



from uuid import UUID


class AssetPosition:
    """
    A simple representation of holding a certain amount of a given asset.
    This is going to be replaced with a much more flexible Portfolio
    representation in Q1'23.
    """

    asset_id: UUID
    """
    Unique identifier of the asset from Serenity's asset master database.
    """

    quantity: float
    """
    The number of tokens, shares, contracts, etc. held in this position.
    If positive this indicates a long position; if negative, a short one.
    """



from uuid import UUID

from serenity_types.refdata.asset import Asset


class Index(Asset):
    provider_org_id: UUID
    """
    The organization that publishes this index, e.g. S&P. Must be of type INDEX_PROVIDER.
    """


class ReferenceIndex(Index):
    """
    An index that tracks or references an exposure. Typically used for settlement purposes
    in derivatives contracts.
    """
    referenced_exposure_id: UUID



from datetime import date
from enum import Enum
from typing import Dict, List, Optional
from uuid import UUID

import pytz

from serenity_types.portfolio.core import AssetPosition
from serenity_types.pricing.core import PricingContext
from serenity_types.utils.serialization import CamelModel


class CashTreatment(Enum):
    """
    For purposes of cash positions, what counts as cash?
    """
    FIAT_ONLY = 'FIAT_ONLY'
    """
    Include only traditional currencies, e.g. USD or EUR.
    """

    FIAT_PEGGED_STABLECOINS = 'FIAT_PEGGED_STABLECOINS'
    """
    Include both fiat and fiat pegs, e.g. USD and USDT.
    """


class MarkTime(Enum):
    """
    For purposes of closing prices in a 24x7 market, which regional market closing should be used?
    """

    NY_EOD = 'NY_EOD'
    """
    Mark prices at 4:30PM America/New_York.
    """

    LN_EOD = 'LN_EOD'
    """
    Mark prices at 4:30PM Europe/London.
    """

    HK_EOD = 'HK_EOD'
    """
    Mark prices at 4:00PM Asia/Hong_Kong.
    """

    UTC = 'UTC'
    """
    Mark prices at UTC midnight.
    """


MARK_TIME_TZ = {
    MarkTime.NY_EOD: pytz.timezone('America/New_York'),
    MarkTime.LN_EOD: pytz.timezone('Europe/London'),
    MarkTime.HK_EOD: pytz.timezone('Asia/Hong_Kong'),
    MarkTime.UTC: pytz.utc
}


class PositionValue(CamelModel):
    """
    Value of a single position in the portfolio.
    """

    value: float
    """
    The value of this position in base currency, e.g. qty * price for simple asset types.
    """

    price: float
    """
    The price for this position according to the chosen mark time.
    """

    qty: float
    """
    The quantity of assets, e.g. 0.7 BTC.
    """

    weight: float
    """
    The weight of this position in the overall portfolio, as a fraction, e.g. 0.12.
    This is just the position's value divided by the portfolio value.
    """


class PositionValuation(CamelModel):
    """
    Close, previous and current values of a single position in the portfolio.
    """

    close: PositionValue
    """
    The value of the asset at the MarkTime, as of the most recent close.
    """

    previous: PositionValue
    """
    The value of the asset at the MarkTime, as of the previous close.
    """

    current: PositionValue
    """
    The value of the position as of the current moment. Requires real-time
    data that will be connected in Q1'23.
    """


class PortfolioValue(CamelModel):
    """
    Total value of the portfolio as of a certain time.
    """

    net_holdings_value: float
    """
    The sum of the values of all non-cash positions.
    """

    gross_holdings_value: float
    """
    The sum of the absolute values of all non-cash positions.
    """

    cash_position_value: float
    """
    The fiat position or stablecoin equivalent based on settings determining whether stablecoins count as cash.
    """

    net_asset_value: float
    """
    NAV, i.e. net_position_value + cash_position_value.
    """


class PortfolioValuationRequest(CamelModel):
    """
    Request to do a NAV calculation for a portfolio. This is simple right now,
    but once we support non-linear assets we will need to extend it.
    """

    portfolio: List[AssetPosition]
    """
    Basic, moment-in time image of the portfolio to be valued.
    """

    pricing_context: PricingContext
    """
    Common settings related to how to value the portfolio, e.g. which prices to load.
    """


class PortfolioValuationResponse(CamelModel):
    """
    Response with the value of the portfolio at top level plus all position values.
    """

    request_id: UUID
    """
    The original request ID sent in. DEPRECATED, will move to higher level.
    """

    as_of_date: date
    """
    The close date on which the portfolio was valued. DEPRECATED, duplicate field.
    """

    pricing_context: PricingContext
    """
    The context that was used to value this portfolio. DEPRECATED, not needed.
    """

    close: PortfolioValue
    """
    The value of the whole portfolio as of the most recent close date.
    """

    previous: PortfolioValue
    """
    The value of the whole portfolio as of the previous close date.
    """

    current: PortfolioValue
    """
    The value of the whole portfolio as of the current moment. Requires real-time
    data that will be connected in Q1'23.
    """

    positions: Dict[UUID, PositionValue]
    """
    The values of each of the individual positions in the portfolio.
    """

    warnings: Optional[List[str]]
    """
    An optional list of warnings, e.g. regarding missing price data.
    """



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src/python'}

packages = \
['serenity_types',
 'serenity_types.catalog',
 'serenity_types.marketdata',
 'serenity_types.platform',
 'serenity_types.portfolio',
 'serenity_types.pricing',
 'serenity_types.pricing.derivatives',
 'serenity_types.pricing.derivatives.options',
 'serenity_types.pricing.derivatives.rates',
 'serenity_types.refdata',
 'serenity_types.risk',
 'serenity_types.utils',
 'serenity_types.valuation']

package_data = \
{'': ['*']}

install_requires = \
['humps>=0.2.2,<0.3.0', 'pydantic>=1.9.1,<2.0.0', 'pytz>=2022.1,<2023.0']

setup_kwargs = {
    'name': 'serenity.types.python',
    'version': '0.0.18',
    'description': 'Python shared types for the Serenity digital asset risk API',
    'long_description': '# serenity.types.python\n\nThis base repository contains shared classes and enumerations common to the Serenity platform and the public SDK.\nIt is not meant to be used or installed on a standalone basis and is intended for use by Serenity clients only.\n',
    'author': 'Cloudwall Support',
    'author_email': 'support@cloudwall.tech',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4',
}


setup(**setup_kwargs)



from enum import Enum
from uuid import UUID

from serenity_types.utils.serialization import CamelModel


class OrganizationType(Enum):
    """
    Category of legal entity; generally used when referring to who provides
    information, prices, positions, etc..
    """

    EXCHANGE = 'EXCHANGE'
    """
    A centralized exchange, e.g. Binance or Deribit.
    """

    DEALER = 'DEALER'
    """
    A broker/dealer, prime broker, OTC desk associated with a prop trading
    firm or market maker, or any other entity that does OTC deals and otherwise
    could be a counterparty.
    """

    INDEX_PROVIDER = 'INDEX_PROVIDER'
    """
    A benchmark or other index provider like S&P.
    """

    DATA_PROVIDER = 'DATA_PROVIDER'
    """
    A data vendor like Kaiko or Amberdata. This is to allow for the possibility
    that data might come either directly from an exchange or indirectly via
    a data aggregator of some kind.
    """

    ORACLE = 'ORACLE'
    """
    A price oracle like ChainLink or Pyth.
    """

    DAO = 'DAO'
    """
    A Decentralized Autonomous Organization (DAO); generally associated with DeFi
    and other protocols as the owner, e.g. MakerDAO.
    """

    OTHER = 'OTHER'
    """
    Any other kind of organization we wish to track.
    """


class Organization(CamelModel):
    organization_id: UUID
    """
    Unique ID for this organization.
    """

    organization_type: OrganizationType
    """
    Catagory of organization, e.g. an exchange.
    """

    short_name: str
    """
    One-word, lowercase code for this organization, e.g. spdix.
    """

    display_name: str
    """
    Human-friend name for this organization, e.g. S&P Dow Jones Indices.
    """



from enum import Enum
from uuid import UUID

from serenity_types.utils.serialization import CamelModel


class NetworkType(Enum):
    """
    Categories of decentralized network.
    """

    BLOCKCHAIN = 'BLOCKCHAIN'
    """
    An L1 blockchain network, e.g. Ethereum.
    """

    L2_PROTOCOL = 'L2_PROTOCOL'
    """
    An L2 (Layer 2) network like Optimism or Arbitrum.
    """

    BRIDGE = 'L2_PROTOCOL'
    """
    A bridge that connects other networks like the Wormhole bridge.
    """


class Network(CamelModel):
    """
    Reference data describing a decentralized network protocol, e.g. an L1 blockchain or bridge.
    """

    network_id: UUID
    """
    Unique ID for this blockchain, bridge, etc..
    """

    network_type: NetworkType
    """
    Category of decentralized network.
    """

    short_name: str
    """
    One-word, lowercase code for this Network, e.g. bitcoin.
    """

    display_name: str
    """
    Human-friendly name for this Network, e.g. Bitcoin.
    """



from pydantic import BaseModel
from humps.camel import case
from enum import Enum
from typing import (AbstractSet,
                    Mapping,
                    Any,
                    Callable,
                    Optional,
                    Union,
                    cast)


IntStr = Union[int, str]
AbstractSetIntStr = AbstractSet[IntStr]
MappingIntStrAny = Mapping[IntStr, Any]


def serenity_json_format(data):
    for k, v in data.items():
        if isinstance(v, dict):
            data[k] = serenity_json_format(v)
        # Can add further JSON formatting needs here in future
        if issubclass(type(v), Enum):
            data[k] = v.name
    return data


def serenity_json(
    self,
    *,
    include: Union['AbstractSetIntStr', 'MappingIntStrAny'] = None,
    exclude: Union['AbstractSetIntStr', 'MappingIntStrAny'] = None,
    by_alias: bool = False,
    exclude_unset: bool = False,
    exclude_defaults: bool = False,
    exclude_none: bool = False,
    encoder: Optional[Callable[[Any], Any]] = None,
    models_as_dict: bool = True,
    **dumps_kwargs: Any,
) -> str:
    encoder = cast(Callable[[Any], Any], encoder or self.__json_encoder__)
    data = dict(
        self._iter(
            to_dict=models_as_dict,
            by_alias=by_alias,
            include=include,
            exclude=exclude,
            exclude_unset=exclude_unset,
            exclude_defaults=exclude_defaults,
            exclude_none=exclude_none,
        )
    )
    if self.__custom_root_type__:
        data = data["__root__"]
    data = serenity_json_format(data)
    return self.__config__.json_dumps(data, default=encoder, **dumps_kwargs)


class CamelModel(BaseModel):
    """
    Helper base class that ensures JSON is encoded camel-case and enums are properly encoded.
    """

    class Config:
        alias_generator = case
        allow_population_by_field_name = True
        arbitrary_types_allowed = True

    json = serenity_json
